#include <p18f4431.h>
#include "main.h"

#pragma config OSC=IRCIO, PWM4MX = RB5  //use internal oscillator, multiplex PWM4 pin onto RB5

void main(void){
	OSCCON=	0b11111111;	//8Mhz oscillator
	TRISA=	0b00000001;	//Set pin RA0 to input
	ANSEL0=	0b00000001;	//Set RA0 to analog
	
	//initialize everything else to output low to avoid floating inputs
	ANSEL1= 0;
	LATA  =	0;
	TRISB = 0;
	LATB  = 0;
	TRISC = 0;
	LATC  = 0;
	TRISD = 0;
	LATD  = 0;
	TRISE = 0;
	LATE  = 0;
	
	//ADC setup
	ADCON0	= 0b00000001;	//single-shot conversion on ADC group A
	ADCON1	= 0b00000000;	//Use AVdd and AVss for references, don't use FIFO buffer
	ADCON2	= 0b10010100;	//R-justified, 4TAD aquisition time, TAD=2*TOSC, where TOSC=0.5 us (clock = FOSC/4)
	ADCON3	= 0b00000000;	//no triggers
	ADCHS	= 0b00000000;	//Use AN0
	
	//PWM Setup
	PTPERH	= 0b00000000;	
	PTPERL	= 0b11111110;	//Use 10-bit resolution for PWM period (PTPER is 8 bits, plus 2 bits of Q-clock match)
//We use 0b11111110 instead of 0b11111111 for PTPERL, or the PWM output would never be able 
//to achieve 100% duty cycle.  If PDC0 and PTPER are equal, then the output would be
//set low when PTMR is 0b11111111 and the two Q-clock bits are 0b11.  The output would
//be set back high the next clock cycle when PTMR goes back to 0, so even with the maximum
//duty cycle, the output would be low for one clock cycle. 
//(see section 17.6 in the PIC18F4431 datasheet)
							
	PTCON0	= 0b00000000;	//PWM Time base 1:1 postscale, FOSC/4 for input, free running mode
	PTCON1	= 0b10000000;	//Enable PWM time base
	PWMCON0	= 0b00010000;	//Enable PWM1 only, Independant/Complementary mode has no effect for only 1 pin
	PWMCON1	= 0b00000000;	//No special event triggers

	while(1){
		ADCON0bits.GO_DONE=1;		//Start A/D conversion process
		while(ADCON0bits.GO_DONE){}	//wait for process to finish
		PDC0H = ADRESH;		//Set PWM duty cycle equal ADC result
		PDC0L = ADRESL;
	}
}	//end main()